#!/usr/bin/python -tt
#
#    Yum Exteder (yumex) - A GUI for yum
#    Copyright (C) 2005 Tim Lauridsen < tla<AT>rasmil<DOT>DK > 
# 
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU Library General Public License as published by
# the Free Software Foundation; version 2 only
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Library General Public License for more details.
#
# You should have received a copy of the GNU Library General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
# $Id: yum24XBase.py,v 1.4 2006/02/15 10:26:28 timlau Exp $

import yum
import yum.Errors 
from yum.packages import *
from yum.logger import Logger
from yumex.yumexi18n import _
import yumex
from repomd import mdUtils, mdErrors
from repomd.packageSack import ListPackageSack
from rpmUtils.miscutils import compareEVR



class yumexBase24(yumex.yumexBase):
    
    '''
     This class is a special yumexBase to make yumex work with yum 2.4.x as it was 
     yum 2.5.x, it contains some of the new methods copied from yum 2.5.x.
     
     Inherence:
     yumexBase24 <- yumex.yumexBase <- yumgui.Base <- yum.YumBase
    '''
    
    def __init__( self, yumexconf, logger, progress=None, loglevel=2 ):
        yumex.yumexBase.__init__( self, yumexconf, logger, progress, loglevel)
        InstallError = UpdateError = yum.Errors.YumBaseError
        self.localPackages = []
        
    def doConfigSetup(self):
        yum.YumBase.doConfigSetup(self)
        conf = self.conf
        self.conf = ConfigWrapper(conf)
    
    
    def install(self, po):
        tx_return = []
        txmbr = self.tsInfo.addInstall(po)
        tx_return.append(txmbr)        
        return tx_return

    
    def update(self, po=None, **kwargs):
        updates = self.up.getUpdatesTuples()
        if self.conf.obsoletes:
            obsoletes = self.up.getObsoletesTuples(newest=1)
        else:
            obsoletes = []


        tx_return = []
        if not po : # update everything (the easy case)
            self.log(5, 'Updating Everything')
            for (obsoleting, installed) in obsoletes:
                obsoleting_pkg = self.getPackageObject(obsoleting)
                installed_pkg =  YumInstalledPackage(self.rpmdb.returnHeaderByTuple(installed)[0])
                self.tsInfo.addObsoleting(obsoleting_pkg, installed_pkg)
                self.tsInfo.addObsoleted(installed_pkg, obsoleting_pkg)
                                
            for (new, old) in updates:
                txmbrs = self.tsInfo.getMembers(pkgtup=old)

                if txmbrs and txmbrs[0].output_state == 'obsoleted':
                    self.log(5, 'Not Updating Package that is already obsoleted: %s.%s %s:%s-%s' % old)
                else:
                    updating_pkg = self.getPackageObject(new)
                    updated_pkg = YumInstalledPackage(self.rpmdb.returnHeaderByTuple(old)[0])
                    self.tsInfo.addUpdate(updating_pkg, updated_pkg)
            return tx_return
        else:
            for (new, old) in updates:
                if po.pkgtup == new:
                    updated_pkg = YumInstalledPackage(self.rpmdb.returnHeaderByTuple(old)[0])
                    txmbr = self.tsInfo.addUpdate(po, updated_pkg)
                    tx_return.append(txmbr)
        return tx_return
        
        
    def remove(self, po=None, **kwargs):
        tx_return = []       
        txmbr = self.tsInfo.addErase(po)
        tx_return.append(txmbr)
        return tx_return   

    def getGroupData( self, grp ):
        #print '\nGroup: %s' % group
        main = {}
        grp = self.groupInfo.matchGroup(grp)
        if grp:
            group = self.groupInfo.group_by_id[grp]
            list = self.groupInfo.mandatory_pkgs[grp]
            if len( list ) > 0:
                pkgs = []
                for item in list:
                    p,installed = self._get_pkg(item)
                    if len(p)>0: # Package Found
                        pkgs.append((p[0],installed))
                main['mandatory'] =  pkgs
            list = self.groupInfo.default_pkgs[grp]
            if len( list ) > 0:
                pkgs = []
                for item in list:
                    p,installed = self._get_pkg(item)
                    if len(p)>0: # Package Found
                        pkgs.append((p[0],installed))
                main['default'] =  pkgs
            list = self.groupInfo.optional_pkgs[grp]      
            if len( list ) > 0:
                pkgs = []
                for item in list:
                    p,installed = self._get_pkg(item)
                    if len(p)>0: # Package Found
                        pkgs.append((p[0],installed))
                main['optional'] =  pkgs                
        return main

# This method i normally defined in yumgui.Base, but it dont find
# any availible packages in yum 2.4.x, so we have to overload it
# and make it work.
 
    def findPackages( self, inst=True, avail=True,best=False, **kwargs ):
        ''' Find packages there match n,e,r,v,a parameters.
        
        @param inst:     Return installed packages.
        @param avail:    Return Available packages
        @param best:     Return the best available package        
        @param name:     Package name to find
        @param version:  Package version to find
        @param release:  Package release to find
        @param arch:     Package arch to find
        @param epoch:    Package epoch to find
        
        '''

        instpkgs = []
        availpkgs = []
        nevra_dict = self._nevra_kwarg_parse(kwargs)
        if avail:
            name = nevra_dict['name']
            availpkgs = self.searchName(name=name)
            if best and availpkgs:
                availpkgs = self.bestPackagesFromList(availpkgs)
        if inst:
            installed_tuples = self.rpmdb.returnTupleByKeyword(
                        name=nevra_dict['name'], epoch=nevra_dict['epoch'],
                        arch=nevra_dict['arch'], ver=nevra_dict['version'],
                        rel=nevra_dict['release'])        
            for tup in installed_tuples:
                hdr = self.rpmdb.returnHeaderByTuple(tup)[0]
                installed_pkg =  YumInstalledPackage(hdr)
                instpkgs.append(installed_pkg)
            
        return availpkgs+instpkgs
    
    def searchName(self, name=None):        
        '''return list of pkgobjects matching the name requested
            it searches directly in the self.plgSack's sqlite db.
        '''
        
        
        returnList = []
        # Search all repositories            
        for (rep,cache) in self.pkgSack.primarydb.items():
            cur = cache.cursor()
            cur.execute("select * from packages WHERE name = %s " , (name))
            for x in cur.fetchall():
                if (self.pkgSack.excludes[rep].has_key(x.pkgId)):
                    continue
                returnList.append(self.pkgSack.pc(self.pkgSack.db2class(x),rep))
        return returnList

    def _nevra_kwarg_parse(self, kwargs):
            
        returndict = {}
        
        try: returndict['name'] = kwargs['name']
        except KeyError:  returndict['name'] = None

        try: returndict['epoch'] = kwargs['epoch']
        except KeyError: returndict['epoch'] = None

        try: returndict['arch'] = kwargs['arch']
        except KeyError: returndict['arch'] = None
        
        # get them as ver, version and rel, release - if someone
        # specifies one of each then that's kinda silly.
        try: returndict['version'] = kwargs['version']
        except KeyError: returndict['version'] = None
        if returndict['version'] is None:
            try: returndict['version'] = kwargs['ver']
            except KeyError: returndict['version'] = None

        try: returndict['release'] = kwargs['release']
        except KeyError: returndict['release'] = None
        if returndict['release'] is None:
            try: release = kwargs['rel']
            except KeyError: returndict['release'] = None
        
        return returndict
    
    def getGroupPackages(self,grp):
        pkglist = []
        if not self.groupInfo.groupExists(grp):
            self.errorlog(0, _('Warning: Group %s does not exist.') % group)
        else:
            grp = self.groupInfo.matchGroup(grp)
            pkglist = self.groupInfo.mandatory_pkgs[grp]+self.groupInfo.default_pkgs[grp]
        return pkglist
    
    def _bestPackageFromList(self, pkglist):
        """take list of package objects and return the best package object.
           If the list is empty, return None. 
           
           Note: this is not aware of multilib so make sure you're only
           passing it packages of a single arch group."""
        
        
        if len(pkglist) == 0:
            return None
            
        if len(pkglist) == 1:
            return pkglist[0]
        
        mysack = ListPackageSack()
        mysack.addList(pkglist)
        bestlist = mysack.returnNewestByNameArch() # get rid of all lesser vers
        
        best = bestlist[0]
        for pkg in bestlist[1:]:
            if len(pkg.name) < len(best.name): # shortest name silliness
                best = pkg
                continue
            elif len(pkg.name) > len(best.name):
                continue

            # compare arch
            arch = rpmUtils.arch.getBestArchFromList([pkg.arch, best.arch])
            if arch == pkg.arch:
                best = pkg
                continue

        return best

    def bestPackagesFromList(self, pkglist, arch=None):
        """Takes a list of packages, returns the best packages.
           This function is multilib aware so that it will not compare
           multilib to singlelib packages""" 
    
        returnlist = []
        compatArchList = rpmUtils.arch.getArchList(arch)
        multiLib = []
        singleLib = []
        for po in pkglist:
            if po.arch not in compatArchList:
                continue
            elif rpmUtils.arch.isMultiLibArch(arch=po.arch):
                multiLib.append(po)
            else:
                singleLib.append(po)
                
        # we should have two lists now - one of singleLib packages
        # one of multilib packages
        # go through each one and find the best package(s)
        for pkglist in [multiLib, singleLib]:
            best = self._bestPackageFromList(pkglist)
            if best is not None:
                returnlist.append(best)
        
        return returnlist



class ConfigWrapper(object):
    '''
    This Class incapsulates a yum 2.4 conf object and add 
    getattr & setattr methods so it can be accessed as it was a
    yum 2.5 conf object.
    
    value = self.conf.<option>
    self.conf.<option> = value
    
    '''
    
    def __init__(self,conf):
        self._conf = conf
        
    def __getattr__(self,attr):
        #print "[%s]" % attr
        if hasattr(self._conf,attr): 
           return getattr(self._conf,attr)
        else:
           return self._conf.getConfigOption(attr)
        
        
#    def __setattr__(self,attr,value):
#        if attr in self.__dict__.keys():       # obj attribute ? 
#            return self.__dict__.set(attr,value)    
#        elif attr in self._conf.__dict__.keys(): 
#           setattr(self._conf,attr,value)
#        else:
#           self._conf.setConfigOption(attr,value)
        
