#!/usr/bin/python -tt
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU Library General Public License as published by
# the Free Software Foundation; version 2 only
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Library General Public License for more details.
#
# You should have received a copy of the GNU Library General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
# $Id: __init__.py,v 1.13 2006/02/14 14:46:25 timlau Exp $
#    
# Authors:
#    Tim Lauridsen <tla@rasmil.dk>

import yum
import gobject
import gtk
import pango
import sys
import time

from yum.packages import YumInstalledPackage
from yumgui.widgets import TextViewConsole

#
# Classes
#

class Base( yum.YumBase ):
    """
        yum wrapper class for a yum GUI.
    """
    def __init__( self, logger=None ):
        '''Initialize the Logger object.
        
        @param logger: logger class to use, if not definded then create one.
        '''
        yum.YumBase.__init__( self )
        if not logger:
            logger = SimpleLogger( doevents=True )
        self.log = logger.log
        self.errorlog = logger.errorlog

    def isPkgInstalled( self, name = None, epoch = None, version = None, 
                           release = None, arch = None, po = None ):
        
        ''' Check if package is installed (po or Nevra)
        @param name: Package name
        @param epoch: Package epoch
        @param version: Package version
        @param release: Package release
        @param arch: Package arch
        @param po: yum package object
        '''
        if po:
            ( name, epoch, version, release, arch ) = po.returnNevraTuple()
        # Setup rpmdb if self.rpmdb not exist
        if not hasattr( self, 'rpmdb' ):
            self.doRpmDBSetup()
        return self.rpmdb.installed( name = name, epoch = epoch, ver = version, 
                                rel = release, arch = arch ) == 1



    def findPackages( self, inst=True, avail=True,best=False, **kwargs ):
        ''' Find packages there match n,e,r,v,a parameters.
        
        @param inst:     Return installed packages.
        @param avail:    Return Available packages
        @param best:     Return the best available package        
        @param name:     Package name to find
        @param version:  Package version to find
        @param release:  Package release to find
        @param arch:     Package arch to find
        @param epoch:    Package epoch to find
        
        '''

        instpkgs = []
        availpkgs = []
        nevra_dict = self._nevra_kwarg_parse(kwargs)
        if avail:
            availpkgs = self.pkgSack.searchNevra(name=nevra_dict['name'],
                      epoch=nevra_dict['epoch'], arch=nevra_dict['arch'],
                    ver=nevra_dict['version'], rel=nevra_dict['release'])
            if best and availpkgs:
                availpkgs = self.bestPackagesFromList(availpkgs)
        if inst:
            installed_tuples = self.rpmdb.returnTupleByKeyword(
                        name=nevra_dict['name'], epoch=nevra_dict['epoch'],
                        arch=nevra_dict['arch'], ver=nevra_dict['version'],
                        rel=nevra_dict['release'])        
            for tup in installed_tuples:
                hdr = self.rpmdb.returnHeaderByTuple(tup)[0]
                installed_pkg =  YumInstalledPackage(hdr)
                instpkgs.append(installed_pkg)
            
        return availpkgs+instpkgs

    # This one was stolen from pirut :-)))
    def simpleDBInstalled(self, name):
        # Doing this directly in the rpm.TransactionSet instead of using self.rpmdb.installed()
        # speeds things up by 400%
        mi = self.ts.ts.dbMatch('name', name)
        if mi.count() > 0:
            return True
        return False
        
        
        
class SimpleLogger:
    """
    A simple logger class, with selective gtkevent handler.
    """
    def __init__( self, loglvl=2, doevents=True ):
        '''Initialize the Logger object.
        
        @param loglvl: The log level, show only log entryes with a level >= loglevel 
        @param doevents: Handle pending gtk event, every time someting is logged.   
        '''
        
        self.loglevel = loglvl
        self.doEvents = doevents
        
    def log( self, level, msg ):
        ''' write message to log (stdout)
        @param level: Message log level
        @param msg: Message to write to log
        '''
        if self.doEvents:
            doGtkEvents()
        if level <= self.loglevel:
            print >> sys.stdout, msg
            
    def errorlog( self, level, msg ):
        ''' write message to errorlog (stderr)
        @param level: Message log level
        @param msg: Message to write to errorlog
        '''
        if self.doEvents:
            doGtkEvents()
        if level <= 0:
            print >> sys.stderr, "ERROR:", msg
        else:
            print >> sys.stderr, "WARNING:", msg            

            
class TextViewLogger( SimpleLogger,TextViewConsole ):            
    """ 
    Logger class there logs into a gtk.TextView
    """

    def __init__( self, textview, loglvl=2, doevents=True, echo=True, addtime=False ):
        '''Initialize the TextViewLogger object.
        
        @param textview: The gtk.TextView widget to log into.
        @param loglvl: The log level, show only log entryes with a level >= loglevel 
        @param doevents: Handle pending gtk event, every time someting is logged.   
        @param echo: Echo the log entries to stdout,stderr.
        @param addtime: Add timestamp before log entries.
        '''
        SimpleLogger.__init__( self, loglvl, doevents )
        TextViewConsole.__init__(self,textview)
        self.doEvents = doevents
        self.echo = echo
        self.loglevel = loglvl
        self.addtime = addtime
        
    def log( self, level, msg ):
        ''' write message to log 
        @param level: Message log level
        @param msg: Message to write to log
        '''
        # add time stamp ?
        msg = str( msg )
        if self.addtime:
            now = time.strftime( "%H:%M:%S", time.localtime() )
            msg = now +" : "+msg
        # echo the message to stdout ?
        if self.echo:
            SimpleLogger.log( self, level, msg )
        if level <= self.loglevel:
            self.write_line( msg+"\n", self.style_ps1 )
        if self.doEvents:
            doGtkEvents()
            
            
        
    def errorlog( self, level, msg ):
        ''' write message to error log 
        @param level: Message log level
        @param msg: Message to write to errorlog
        '''
        SimpleLogger.errorlog( self, level, msg )
        if level <= self.loglevel:            
            self.write_line( msg+"\n", self.style_err )
        if self.doEvents:
            doGtkEvents()   
    
#
# Functions
#    
    
def busyCursor(mainwin,insensitive=False):
    ''' Set busy cursor in mainwin and make it insensitive if selected '''
    mainwin.window.set_cursor(gtk.gdk.Cursor(gtk.gdk.WATCH))
    if insensitive:
        mainwin.set_sensitive(False)
    doGtkEvents()

def normalCursor(mainwin):
    ''' Set Normal cursor in mainwin and make it sensitive '''
    mainwin.window.set_cursor(None)
    mainwin.set_sensitive(True)        
    doGtkEvents()
    
def doGtkEvents():
    while gtk.events_pending():      # process gtk events
        gtk.main_iteration()
        
    